/* sci.c                                  */
/* H8/3048 Sireal communication interface */
/* H. Okajima                             */
/* Oct. 25 2001                           */

#include "3048fone.h"
#include "defs.h"
#include "sci.h"
#include "ansi.h"

/* SCI channel definition */
#define SCIn SCI1     /*  SCI0 or SCI1 */

/* initialize SCI */
void init_SCI(void)
{
   unsigned int dmy;

   /* Async, 8bits, non-parity, 1 stop-bit, clock=phi/1 */
   SCIn.SMR.BYTE = 0x00;

   /* Tx-Rx interrupt disable, Tx-Rx enable */
   SCIn.SCR.BYTE = 0x30;

   /* Set baurate */
   SCIn.BRR = DEF_BRR;

   /* wait for 1bit transfer time */
   for(dmy = 280 ;dmy > 0 ;dmy--)
      ;

   /* Crear ORER, FER, PER */
   SCIn.SSR.BYTE &= 0x80;
}


/* macro output 1 byte to SCI */
static int _putc_SCI(int ch)
{
   while (!SCIn.SSR.BIT.TDRE)
     ;
   SCIn.TDR = ch;
   /* TxD buffer empty flag clear */
   SCIn.SSR.BIT.TDRE = 0;

   return 0;
}

/* output 1 byte to SCI    */
/* CR --> CR+LF conversion */
int putc_SCI(int ch)
{
   if (ch != '\n') {
      _putc_SCI(ch);
      return ch;
   }
   _putc_SCI(CR);
   _putc_SCI(LF);

   return ch;
}


/* input 1 byte from SCI */
int getc_SCI(void)
{
   int c;

   while (!(SCIn.SSR.BIT.RDRF | SCIn.SSR.BIT.ORER
            | SCIn.SSR.BIT.FER | SCIn.SSR.BIT.PER))
      ;

   if (SCIn.SSR.BIT.RDRF) {
      c = SCIn.RDR;
      SCIn.SSR.BIT.RDRF = 0;   /* RxD flag clear  */
      if (c == CR)
         c = '\n';             /* convert LF code */
      putc_SCI(c);             /* echo back       */

      return c;
   }
   if (SCIn.SSR.BIT.ORER) {
      return E_SCI_ORER;       /* Overrun error */
   }
   if (SCIn.SSR.BIT.FER) {
      return E_SCI_FER;        /* Framing error  */
   }
   if (SCIn.SSR.BIT.PER) {
      return E_SCI_PER;        /* Parity error   */
   }
   return 0;                   /* Unknown error  */
}

